function [sys,x0,str,ts] = heatx(t,x,u,flag,Param)
%
%	This is the sfunction file for the heat
%	exchanger simulation.
%
%	Variable List
%	 Inputs(u vector):
%		Tin			inlet water temperature, deg. C
%		Fhin			inlet hot side flow, gpm
%		Qin			power input to heater, W
%		Fcin			inlet cold side flow, gpm
%
%	 Outputs:
%		Thout			hot side temperature outlet from double pipe, deg. C
%		Thin			hot side temperature inlet to double pipe, deg. C
%		Thm   		hot side temperature in double pipe compartment 3 
%						from the hot inlet (middle compartment), deg. C
%		Tcout			cold side temperature outlet from double pipe, deg. C
%
%	 State(x vector):
%		Theat			heater temperature, deg. C
%		Thin			hot side temperature inlet to double pipe, deg. C
%		Thm(m)		hot side temperature in double pipe compartment m 
%						from the hot inlet (m goes from 1 to 5), deg. C
%		Tcm(m)		cold side temperature in double pipe compartment m
%						from the hot inlet (m goes from 1 to 5), deg. C
%		Thout			hot side temperature outlet from double pipe, deg. C
%
%	 Physical variables:
%	  These are found in the vector Param which has the following entries:
%
%		Param(1)		tstep			time step used in the calculations, s
%		Param(2)		Cpw			specific heat of water, cal/(g deg. C)
%		Param(3)		rhow			density of water, g/mL
%		Param(4)		Cpheat		specific heat of heater, cal/(g deg. C)
%		Param(5)		Mheat			mass of heater, g
%		Param(6)		rhoheat		densty of heater, g/mL
%		Param(7)		Uheat			overall heat transfer coefficient for the heater, kW/(m^2 deg. C)
%		Param(8)		Upipe			overall heat transfer coefficient for the double pipe, kW/(m^2 deg. C)
%		Param(9)		oDodpipe		outer diameter of the outer pipe of the double pipe, in.
%		Param(10)	iDodpipe		inner diameter of the outer pipe of the double pipe, in.
%		Param(11)	oDidpipe		outer diameter of the inner pipe of the double pipe, in.
%		Param(12)	iDidpipe		inner diameter of the inner pipe of the double pipe, in.
%		Param(13)	Lpipe			length of the double pipe exchanger, in.
%		Param(14)	oDhpipe		outer diameter of the pipe surrounding the heater, in.
%		Param(15)	iDhpipe		inner diameter of the pipe surrounding the heater, in.
%		Param(16)	Dheat			diameter of the heater, in.
%		Param(17)	Lheat			length of the heater, in.
%
%	 Calculation variables
%		qold			old value of q in the iteration procedure on s.s. T's
%		qnew			new value of q in the iteration procedure on s.s. T's
%		eps			minimum error in q required for iteration to stop
%		q2				intermediate value of q used in the iteration procedure
%		i				iteration counter
%		tcm_last		the calculated value of the inlet cold water to the double pipe
%						(to be compared with Tin)
%
switch flag,
  case 0, % Initialization
    [sys,x0,str,ts]=mdlInitializeSizes(u,Param);
  case 1, % For continuous states which we don't have
    sys=mdlDerivatives(t,x,u);
  case 2, % Updating the discrete states
     sys=mdlUpdate(t,x,u,Param);
  case 3, % Computing the output vector
    sys=mdlOutputs(t,x,u);
  case 4, % Time of next hit
    sys=mdlGetTimeOfNextVarHit(t,x,u);
  case 9, % Finished
    sys=mdlTerminate(t,x,u);
  otherwise % Invalid input
    error(['Unhandled flag = ',num2str(flag)]);
end
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes(u,Param)
%
% call simsizes for a sizes structure, fill it in and convert it to a
% sizes array.
%
% Note that in this example, the values are hard coded.  This is not a
% recommended practice as the characteristics of the block are typically
% defined by the S-function parameters.
%
sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 13;
sizes.NumOutputs     = 4;
sizes.NumInputs      = 4;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;   % at least one sample time is needed
sys = simsizes(sizes);

%
% initialize the initial conditions
% This is where I call the function file to find the initial s.s. values
[x1,x2,x3,x4,x5]  = steady([20,0.1,250,0.1],Param);
x0  = [x1,x2,x3,x4,x5];
%
% str is always an empty matrix
%
str = [];
%
% initialize the array of sample times
%
ts  = [Param(1),0];

% end mdlInitializeSizes
%
%=============================================================================
% mdlDerivatives
% Return the derivatives for the continuous states.
%=============================================================================
%
function sys=mdlDerivatives(t,x,u)
% There are no continuous variables so this is all that it needs to do.
sys = [];


% end mdlDerivatives
%
%=============================================================================
% mdlUpdate
% Handle discrete state updates, sample time hits, and major time step
% requirements.
%=============================================================================
%
function sys=mdlUpdate(t,x,u,Param)
% This updates the state vector by using the discrete PDE's
[s1,s2,s3,s4,s5] = update(u,x,Param);
sys = [s1,s2,s3,s4,s5];
% end mdlUpdate
%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u)
% This puts in the outputs.
sys = [x(13),x(2),x(10),x(8)];
% end mdlOutputs
%
%=============================================================================
% mdlGetTimeOfNextVarHit
% Return the time of the next hit for this block.  Note that the result is
% absolute time.  Note that this function is only used when you specify a
% variable discrete-time sample time [-2 0] in the sample time array in
% mdlInitializeSizes.
%=============================================================================
%
function sys=mdlGetTimeOfNextVarHit(t,x,u)
sys = [];
% end mdlGetTimeOfNextVarHit
%
%=============================================================================
% mdlTerminate
% Perform any end of simulation tasks.
%=============================================================================
%
function sys=mdlTerminate(t,x,u)
sys = [];
% end mdlTerminate
